/**********************************************************************/
/*                                                                    */
/* RexxMail installation program.                                     */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program forms part of the RexxMail package, and may not be    */
/* distributed separately.                                            */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use RexxComm.CMD to reformat the comments in this program.         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* The latest versions of RexxMail and RexxComm can be found at       */
/*               www.degeus.com/rexx                                  */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program is released under the terms of the GNU license, see   */
/*               www.gnu.org/copyleft/gpl.html                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* (c) 2000-2001 Marcus C. de Geus                                    */
/*               marcus@degeus.com                                    */
/*               www.degeus.com                                       */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* History:                                                           */
/*                                                                    */
/* 2001-04-03: Corrected "chaining error" bug when using CMD.EXE.     */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use it if you like it. Don't if you don't. No legalese.            */
/*                                                                    */
/**********************************************************************/

/**********************************************************************/
/* Some general bits                                                  */
/**********************************************************************/

signal on halt  /* handle halt condition */
signal on syntax  /* handle bad syntax */

/**********************************************************************/
/* Define the program directory                                       */
/**********************************************************************/

parse source . . ProgSpec  /* get the program name */
ProgDir = filespec('D',ProgSpec)||filespec('P',ProgSpec)  /* get the program directory */

/**********************************************************************/
/* Start a log file                                                   */
/**********************************************************************/

LogFile = ProgDir||'install.log'  /* define the log file name */
Message = date('S')||' '||time('N')||,  /* include a date and time stamp */
          ' : Installation started'  /* end of the message to log */
call WriteLog LogFile,Message  /* report */

if (\LoadRexxUtils()) then  /* if we cannot use or load the Rexx utility functions */
do
 call Halt  /* exit */
end

call Explain  /* Explain to the user what's going to happen */

/**********************************************************************/
/* Check the program directory name for spaces                        */
/**********************************************************************/

if (pos(' ',ProgDir) > 0) then  /* if the program directory name contains one or more spaces */
do
 Message =  'The program directory name "'||ProgDir||,  /* the message to log */
            '" contains blank spaces'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
 Message = 'Please rename the program directory'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
 call Halt  /* and abort */
end

/**********************************************************************/
/* Define the current user name                                       */
/**********************************************************************/

UserName = value('USER',,'OS2ENVIRONMENT')  /* look for a user name */

if (UserName = '') then  /* if user name is found */
do
 UserName = 'DEFAULT'  /* use the default value */
end

/**********************************************************************/
/* Get the main mail folder location                                  */
/**********************************************************************/

MailLoc = GetLoc()  /* get the location for the Rexxmail mail folder */

/**********************************************************************/
/* Define the program subdirectories                                  */
/**********************************************************************/

OrgDir = ProgDir||'Originals'  /* define the configuration files source dir in the program directory */
IconsDir = ProgDir||'icons'  /* define the icon files source dir in the program directory */

/**********************************************************************/
/* Check a few directories                                            */
/**********************************************************************/

FileList = '*DISPCOLL.CMD*',  /* a file to look for */
           '*LOCATION.TXT*',  /* a file to look for */
           '*MMENCODE.EXE*',  /* a file to look for */
           '*MMENCODE.TXT*',  /* a file to look for */
           '*MUNPACK.EXE*',  /* a file to look for */
           '*MUNPACK.TXT*',  /* a file to look for */
           '*README.TXT*',  /* a file to look for */
           '*REXXMAIL.CMD*',  /* a file to look for */

call CheckDir strip(ProgDir,'T','\'),FileList  /* check the program dir itself */

FileList = '*CLOSING.TXT*',  /* a file to look for */
           '*INDEX.HTML*',  /* a file to look for */
           '*MAILADDR.TXT*',  /* a file to look for */
           '*MAILREAD.TXT*',  /* a file to look for */
           '*MAILSENT.TXT*',  /* a file to look for */
           '*MIMETYPE.TXT*',  /* a file to look for */
           '*REXXMAIL.GIF*',  /* a file to look for */
           '*SETTINGS.TXT*'  /* a file to look for */

call CheckDir OrgDir,FileList  /* check the original files dir */

FileList = '*MAILADDR.ICO*',  /* a file to look for */
           '*MAILADEX.ICO*',  /* a file to look for */
           '*MAILADIM.ICO*',  /* a file to look for */
           '*MAILDISP.ICO*',  /* a file to look for */
           '*MAILINFO.ICO*',  /* a file to look for */
           '*MAILOPEN.ICO*',  /* a file to look for */
           '*MAILWARN.ICO*',  /* a file to look for */
           '*MAIL_ACC.ICO*',  /* a file to look for */
           '*MAIL_CFG.ICO*'  /* a file to look for */

call CheckDir IconsDir,FileList  /* check the Icons dir */

FileList = '*00000000.ICO*',  /* a file to look for */
           '*00010000.ICO*',  /* a file to look for */
           '*01000000.ICO*',  /* a file to look for */
           '*01010000.ICO*',  /* a file to look for */
           '*01100000.ICO*',  /* a file to look for */
           '*01110000.ICO*',  /* a file to look for */
           '*10000000.ICO*',  /* a file to look for */
           '*10100000.ICO*',  /* a file to look for */

PaperDir = IconsDir||'\blue_white'  /* the minimum stationery icons dir */
call CheckDir PaperDir,FileList  /* check the Icons dir */

/**********************************************************************/
/* Create a few subdirectories if they do not exist                   */
/**********************************************************************/

ObjectsDir = ProgDir||'Objects'  /* define the Objects dir in the program directory */
call MakeDir ObjectsDir  /* make the dir if necessary */

TempDir = ProgDir||'Temp'  /* define the Temp dir in the program directory */
call MakeDir TempDir  /* make the dir if necessary */

InfoDir = ProgDir||'Information'  /* define the Info dir in the program directory */
call MakeDir InfoDir  /* make the dir if necessary */

/**********************************************************************/
/* Create the main folder objects                                     */
/**********************************************************************/

Settings = 'ALWAYSSORT=YES;'||,  /* always sort the next folders */
           'WORKAREA=NO;'||,  /* do not make the next folders work areas */
           'DEFAULTVIEW=ICON;'||,  /* use icon views for the next folders */
           'ICONVIEW=NORMAL,NONGRID;'||,  /* use standard view for these folders */
           'NORENAME=YES;'||,  /* the next folders cannot be renamed */
           'NOTVISIBLE=NO;'||,  /* the next folders are visible (in the main folder) */
           'DEFAULTSORT=-2;'  /* sort the next folders' contents by name */

AddSettings = 'ICONFILE='||IconsDir||'\Mail_Acc.ICO;'  /* additional settings for the Accessories folder */
call MakeFolder 'Accessories',MailLoc,Settings||AddSettings  /* make the Accessories folder */

AddSettings = 'ICONFILE='||IconsDir||'\Mail_Cfg.ICO;'  /* additional settings for the Configuration folder */
call MakeFolder 'Configuration', MailLoc,Settings||AddSettings  /* make the Configuration folder */

Settings = 'ALWAYSSORT=YES;'||,  /* always sort the next folders */
           'WORKAREA=YES;'||,  /* make the next folders work areas */
           'DEFAULTVIEW=ICON;'||,  /* use icon views for the next folders */
           'ICONVIEW=NORMAL,FLOWED;'||,  /* use flowed (multi-column) view for the folders */
           'NORENAME=YES;'||,  /* the next folders cannot be renamed */
           'NOTVISIBLE=YES;'||,  /* the next folders are invisible (in the main folder) */
           'DEFAULTSORT=-2'  /* sort the next folders' contents by name */

AddSettings = 'ICONFILE='||IconsDir||'\MailAddr.ICO;'  /* additional settings for the Addresses folder */
call MakeFolder 'Addresses',MailLoc,Settings||AddSettings  /* make the Addresses folder */

call MakeFolder 'In',MailLoc,Settings  /* make the In folder, no special icon */
call MakeFolder 'In Archive',MailLoc,Settings  /* make the In Archive folder, no special icon */
call MakeFolder 'Out',MailLoc,Settings  /* make the Out folder, no special icon */
call MakeFolder 'Out Archive',MailLoc,Settings  /* make the Out Archive folder, no special icon */

Settings = 'ALWAYSSORT=YES;'||,  /* always sort the next folders */
           'WORKAREA=YES;'||,  /* make the next folders work areas */
           'DEFAULTVIEW=ICON;'||,  /* use icon views for the next folders */
           'ICONVIEW=NORMAL,FLOWED;'||,  /* use flowed (multi-column) view for the folders */
           'NORENAME=NO;'||,  /* the next folders can be renamed */
           'NOTVISIBLE=NO;'||,  /* the next folders are visible */
           'DEFAULTSORT=-2'  /* sort the next folders' contents by name */

call MakeFolder 'Read',MailLoc||'\In Archive',Settings  /* make the Read folder */
call MakeFolder 'Sent',MailLoc||'\Out Archive',Settings  /* make the Sent folder */
call MakeFolder 'RexxMail',MailLoc||'\In Archive',Settings  /* make the incoming RexxMail folder */
call MakeFolder 'RexxMail',MailLoc||'\Out Archive',Settings  /* make the outgoing RexxMail folder */

/**********************************************************************/
/* Copy original files to the Information folder                      */
/**********************************************************************/

call sysfiletree OrgDir||'\*.html','Files.','FO'  /* look for HTML files in the original files source directory */
do Index = 1 to Files.0  /* take each of the files we found */
 if (syscopyobject(Files.Index,InfoDir)) then  /* if we can copy it to the Information folder */
 do
  Message = 'Copied file "'||Files.Index||'" to "'||InfoDir||'"'  /* the message to log */
  call WriteLog LogFile,Message  /* report */
 end
 else  /* if we cannot copy the file to the Information folder */
 do
  Message = 'ERROR: Could not copy file "'||Files.Index||,  /* the message to log */
            '" to "'||InfoDIr||'", file may already exist'  /* the message to log */
  call WriteLog LogFile,Message  /* report */
 end
end

call sysfiletree OrgDir||'\*.gif','Files.','FO'  /* look for GIF files in the original files source directory */
do Index = 1 to Files.0  /* take each of the files we found */
 if (syscopyobject(Files.Index,InfoDir)) then  /* if we can copy it to the Information folder */
 do
  Message = 'Copied file "'||Files.Index||'" to "'||InfoDir||'"'  /* the message to log */
  call WriteLog LogFile,Message  /* report */
 end
 else  /* if we cannot copy the file to the Information folder */
 do
  Message = 'ERROR: Could not copy file "'||Files.Index||,  /* the message to log */
            '" to "'||InfoDIr||'", file may already exist'  /* the message to log */
  call WriteLog LogFile,Message  /* report */
 end
end

/**********************************************************************/
/* Copy original files to the Configuration folder                    */
/**********************************************************************/

call sysfiletree OrgDir||'\*.txt','Files.','FO'  /* look for files in the original files source directory */
do Index = 1 to Files.0  /* take each of the files we found */
 if (syscopyobject(Files.Index,MailLoc||'\Configuration')) then  /* if we can copy it to the Configuration folder */
 do
  Message = 'Copied file "'||Files.Index||,  /* the message to log */
            '" to "'||MailLoc||'\Configuration"'  /* the message to log */
  call WriteLog LogFile,Message  /* report */
 end
 else  /* if we cannot copy the file to the Configuration folder */
 do
  Message = 'ERROR: Could not copy file "'||Files.Index||,  /* the message to log */
            '" to "'||MailLoc||'\Configuration", file may already exist'  /* the message to log */
  call WriteLog LogFile,Message  /* report */
 end
end

/**********************************************************************/
/* Create the program objects in the program objects folder           */
/**********************************************************************/
/* Note strange order of calling MakeObject to make sure the program- */
/* file associations are created in the right order of preference.    */
/**********************************************************************/

Location = ObjectsDir  /* the objects will be created in the Objects sub dir of the program dir */

Settings = 'MINIMIZED=NO;'||,  /* do not start the program minimized */
           'NOAUTOCLOSE=NO;'||,  /* close the program session after completion */
           'PROGTYPE=WINDOWABLEVIO;'||,  /* use a VIO (OS/2) window */
           'STARTUPDIR='||ProgDir  /* start the program in the RexxMail program directory */

ExeName = ProgDir||'maillock.cmd'  /* the following object uses maillock.cmd in the program directory */
Title = 'RexxMail - Lock/Unlock Mail Folders'  /* the title to use */
ObjectID = '<REXXMAIL_LOCK>'  /* the object id to use */
IconFile = IconsDir||'\MailLock.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Settings = 'MINIMIZED=YES;'||,  /* start the program minimized */
           'NOAUTOCLOSE=NO;'||,  /* close the program session after completion */
           'PROGTYPE=WINDOWABLEVIO;'||,  /* use a VIO (OS/2) window */
           'STARTUPDIR='||ProgDir  /* start the program in the RexxMail program directory */

AssocType = 'Mail Message Out'  /* the association to use for the next few objects */

ExeName = ProgDir||'rexxmail.cmd'  /* the following object uses rexxmail.cmd in the program directory */
Title = 'RexxMail - Edit Message'  /* the title to use */
ObjectID = '<REXXMAIL_EDIT>'  /* the object id to use */
Parameters = '/Edit'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

ExeName = '*'  /* use the default, i.e. the command processor */
Title = 'RexxMail - Dispatch'  /* the title to use */
ObjectID = '<REXXMAIL_DISPATCH>'  /* the object id to use */
Parameters = '/c start /c /min '||ProgDir||'dispcoll.cmd'  /* start a new session running the DispColl.CMD program */
IconFile = IconsDir||'\MailDisp.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = 'Mail Message In'  /* the associations to use */

ExeName = ProgDir||'rexxmail.cmd'  /* the following objects all share the same executable name, i.e. rexxmail.cmd in the program directory */

Title = 'RexxMail - View Message'  /* the title to use */
ObjectID = '<REXXMAIL_VIEW>'  /* the object id to use */
Parameters = '/View'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - View Raw Message'  /* the title to use */
ObjectID = '<REXXMAIL_RAW>'  /* the object id to use */
Parameters = '/Raw'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = 'Mail Message In,Mail Message Out'  /* the associations to use */

Title = 'RexxMail - Address'  /* the title to use */
ObjectID = '<REXXMAIL_ADDRESS>'  /* the object id to use */
Parameters = '/Address'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailAddr.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = 'Mail Message Out'  /* the association to use for the next few objects */

Title = 'RexxMail - Set ASCII'  /* the title to use */
ObjectID = '<REXXMAIL_SETASCII>'  /* the object id to use */
Parameters = '/SetASCII'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Set Quoted-Printable'  /* the title to use */
ObjectID = '<REXXMAIL_SETNONASCII>'  /* the object id to use */
Parameters = '/SetNonASCII'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = 'Mail Message In'  /* the associations to use */

Title = 'RexxMail - Reply'  /* the title to use */
ObjectID = '<REXXMAIL_REPLY>'  /* the object id to use */
Parameters = '/Reply'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Reply to All'  /* the title to use */
ObjectID = '<REXXMAIL_REPLYTOALL>'  /* the object id to use */
Parameters = '/ReplyToAll'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Reply to Sender'  /* the title to use */
ObjectID = '<REXXMAIL_REPLYTOSENDER>'  /* the object id to use */
Parameters = '/ReplyToSender'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Reply to Recipients'  /* the title to use */
ObjectID = '<REXXMAIL_REPLYTORECIPIENTS>'  /* the object id to use */
Parameters = '/ReplyToRecipients'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

AssocType = ''  /* no associations for the next ones */

Title = 'RexxMail - Export Addresses'  /* the title to use */
ObjectID = '<REXXMAIL_EXPORT>'  /* the object id to use */
Parameters = '/Export'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailAdEx.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Import Addresses'  /* the title to use */
ObjectID = '<REXXMAIL_IMPORT>'  /* the object id to use */
Parameters = '/Import'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailAdIm.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Info'  /* the title to use */
ObjectID = '<REXXMAIL_INFO>'  /* the object id to use */
Parameters = '/Info'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailInfo.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

Title = 'RexxMail - Open Mail Folders'  /* the title to use */
ObjectID = '<REXXMAIL_OPEN>'  /* the object id to use */
Parameters = '/Open'  /* the RexxMail switch to use */
IconFile = IconsDir||'\MailOpen.ICO'  /* the icon file to use */
call MakeObject Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,Settings  /* make the object */

/**********************************************************************/
/* Make shadows of some objects in the main folder                    */
/**********************************************************************/

call MakeShadow '<REXXMAIL_ADDRESS>',MailLoc  /* make a shadow of the Add to Addresses program object */
call MakeShadow '<REXXMAIL_DISPATCH>',MailLoc  /* make a shadow of the Dispatch program object */

/**********************************************************************/
/* Make shadows of some objects in the Accessories folder             */
/**********************************************************************/

call MakeShadow '<REXXMAIL_EXPORT>',MailLoc||'\Accessories'  /* make a shadow of the Export Addresses program object */
call MakeShadow '<REXXMAIL_IMPORT>',MailLoc||'\Accessories'  /* make a shadow of the Import Addresses program object */
call MakeShadow '<REXXMAIL_INFO>',MailLoc||'\Accessories'  /* make a shadow of the Info program object */
call MakeShadow '<REXXMAIL_LOCK>',MailLoc||'\Accessories'  /* make a shadow of the Lock & Unlock Mail Folders program object */

/**********************************************************************/
/* Create the start folder object on the desktop                      */
/**********************************************************************/

StartID = '<REXXMAIL_START>'  /* the object ID of the Start folder */

Settings = 'ALWAYSSORT=YES;'||,  /* always sort the RexxMail Start folder */
           'DEFAULTVIEW=ICON;'||,  /* use icon view in the RexxMail Start folder */
           'OBJECTID='||StartID  /* use this object ID for the RexxMail Start folder */
NewStart = MakeFolder('RexxMail - Start','<WP_DESKTOP>',Settings)  /* if we can make the RexxMail Start folder, NewStart will be true */

/**********************************************************************/
/* Make some shadows in the Start folder                              */
/**********************************************************************/

if (NewSTart) then  /* if the Start folder was newly created */
do
 call MakeShadow '<REXXMAIL_OPEN>',StartID  /* make a shadow of the Open program object */
 call MakeShadow strip(ProgDir,'T','\'),StartID  /* make a shadow of the program dir */
end

/**********************************************************************/
/* Write main location to "location.txt" in the program directory     */
/**********************************************************************/

LocFile = ProgDir||'location.txt'  /* the name of the file which will contain the main dir location */

if (stream(LocFile,'C','OPEN WRITE') >< 'READY:') then  /* if we cannot open the location file */
do
 Message = 'FATAL ERROR: Could not write main mail folder location "'||MailLoc||,  /* the message to log */
           '" to "'||LocFile||'"'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
 call Halt  /* and abort */
end

call lineout LocFile,MailLoc||' = '||UserName||';'  /* write the location and user name to the location file */

call stream LocFile,'C','CLOSE'  /* close the file */

Message = 'Main mail folder location "'||MailLoc||,  /* the message to log */
          '" written to "'||LocFile||'"'  /* the message to log */
call WriteLog LogFile,Message  /* report */

/**********************************************************************/
/* Create default message templates from the 'mailaddr.txt' file      */
/**********************************************************************/

call ClearScreen  /* clear the screen */

call charout StdOut,'Create new message templates? (recommended) [Y/N] Y'||d2c(8)  /* prompt the user */
Reply = sysgetkey('ECHO')  /* get a response */
say ''  /* skip to the next line */
if (pos(Reply,d2c(13)||'Yy+') > 0) then  /* if it is Y(es) or [Enter] */
do
 'call '||ProgDir||'rexxmail.cmd /import'  /* import the default addresses */
end

call InstallOK  /* that's all, folks! */

/**********************************************************************/
/* Get a valid directory location                                     */
/**********************************************************************/

GetLoc: procedure expose LogFile  /* gets RexxMail destination folder */

call ClearScreen  /* clear the screen */

say 'Enter the full drive, path and name for the main directory:'  /* show a message */
parse pull Location  /* get a location from the user */

Location = strip(Location,'B',' ')  /* remove excess spaces */
if (Location = '') then  /* if we have nothing left */
do
 Message = 'No main directory location given'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
 call Halt  /* and abort */
end

Location = strip(Location,'B','\')  /* remove any trailing backslash */
if (Location = '') then  /* if we have nothing left */
do
 Message = 'No main directory entry'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
 call Halt  /* and abort */
end

DriveLetter = filespec('D',Location)  /* get the drive letter */
if (DriveLetter = '') then  /* if we have nothing */
do
 Message = 'No drive letter in main directory entry'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
 Location = ''  /* we have no location */
 call Halt  /* and abort */
end

CurDir = directory()  /* store the current directory name */
if (directory(Location) >< '') then  /* if we can change to the specified location */
do
 call directory CurDir  /* return to the original dir */
 call sysfiletree Location||'\*.*','Stuff.','BO'  /* look for files and directories in the location */
 if (Stuff.0 > 0) then  /* if we found anything */
 do
  say ''  /* empty line */
  say '"'||Location||'" is not empty;'  /* report */
  say ''  /* empty line */
  call charout StdOut,'Use the directory anyway? [Y/N] Y'||d2c(8)  /* user prompt */
  InKey = sysgetkey('ECHO')  /* get a response */
  say ''  /* skip to the next line */
  if (pos(InKey,'Yy+'||d2c(13)) = 0) then  /* if it is not Y(es) or [Enter] */
  do
   Location = ''  /* we have no location */
   call Halt  /* and abort */
  end
 end
end
else  /* if we cannot change to the location */
do
 say ''  /* empty line */
 say '"'||Location||'" does not exist;'  /* report */
 say ''  /* empty line */
 call charout StdOut,'Create a new directory? [Y/N] Y'||d2c(8)  /* user prompt */
 InKey = sysgetkey('ECHO')  /* get a response */
 say ''  /* skip to the next line */
 if (pos(InKey,'Yy+'||d2c(13)) = 0) then  /* if it is not Y(es) or [Enter] */
 do
  Message = 'Creation of directory "'||Location||'" refused by user'  /* the message to log */
  call WriteLog LogFile,Message  /* report */
  Location = ''  /* we have no location */
  call Halt  /* abort */
 end
 else  /* if we can go ahead */
 do
  if (sysmkdir(Location) = 0) then  /* if we created the direcory */
  do
   Message = 'Created directory "'||Location||'"'  /* the message to log */
   call WriteLog LogFile,Message  /* report */
  end
  else  /* if the directory could not be created */
  do
   Message = 'FATAL ERROR: Could not create directory "'||Location||'"'  /* the message to log */
   call WriteLog LogFile,Message  /* report */
   Location = ''  /* we have no location */
   call Halt  /* and abort */
  end
 end
end

Settings = 'ALWAYSSORT=YES;'||,  /* keep the main directory sorted */
           'WORKAREA=YES;'||,  /* make the main directory a work area */
           'DEFAULTVIEW=ICON;'  /* the default view of the main directory is icons */

call syssetobjectdata Location,Settings  /* do it */

return Location  /* return with the location spec */

/**********************************************************************/
CheckDir: procedure expose LogFile  /* checks the existence of a directory */
/**********************************************************************/

parse arg DirName,FileList  /* get the name of the directory to check */

OrgDir = directory()  /* store the current directory name */

if (directory(DirName) = '') then  /* if we cannot change to the directory */
do
 Message = 'ERROR: Could not find directory "'||DirName||'"'  /* the message to log */
 say Message  /* report */
 call charout StdOut,'Continue anyway? [Y/N] N'||d2c(8)  /* user prompt */
 InKey = sysgetkey('ECHO')  /* get a response */
 say ''  /* skip to the next line */
 if (pos(InKey,'Yy+') = 0) then  /* if it is not Y(es) or [+] */
 do
  call WriteLog LogFile,Message  /* report */
  call Halt  /* and abort */
 end
end

call directory OrgDir  /* change back to the original directory */

do while (FileList >< '')  /* while we still have something in the file list */
 parse var FileList '*' NextFile '*' FileList  /* get the next name from the list */
 if (stream(DirName||'\'||NextFile,'C','QUERY EXISTS') = '') then  /* if the file does not exist */
 do
  Message = 'ERROR: Could not find file "'||NextFile||,  /* the message to log */
            '" in "'||DirName||'"'  /* the message to log */
  say Message  /* report */
  call charout StdOut,'Continue anyway? [Y/N] N'||d2c(8)  /* user prompt */
  InKey = sysgetkey('ECHO')  /* get a response */
  say ''  /* skip to the next line */
  if (pos(InKey,'Yy+') = 0) then  /* if it is not Y(es) or [+] */
  do
   call WriteLog LogFile,Message  /* report */
   call Halt  /* and abort */
  end
 end
end

Message = 'Checked directory "'||DirName||'"'  /* the message to log */
call WriteLog LogFile,Message  /* report */

return 0  /* end of CheckDir */

/**********************************************************************/
MakeDir: procedure  /* check for the existence of a dir, and if not found, create it */
/**********************************************************************/

parse arg NewDir  /* get the name of the dir to make */

call sysfiletree NewDir,'Dirs.','DO'  /* look for the dir */
if (Dirs.0 = 0) then  /* if it is not found */
do
 if (sysmkdir(NewDir) >< 0) then  /* if we cannot create the dir */
 do
  Message = 'FATAL ERROR: Could not create directory "'||NewDir||'"'  /* the message to log */
  call WriteLog LogFile,Message  /* report */
  call Halt  /* and abort */
 end
end

return  /* end of MakeDir */

/**********************************************************************/
MakeFolder: procedure expose LogFile  /* creates a folder */
/**********************************************************************/

parse arg Name,Location,Settings  /* get the arguments */

if (syscreateobject('WPFolder',Name,Location,Settings)) then  /* if we can create the folder */
do
 Message = 'Created folder "'||Name||,  /* the message to log */
           '" in "'||Location||'"'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
 Success = 1  /* the result is true */
end
else  /* if we cannot create the object */
do
 Message = 'ERROR: Could not create folder "'||Name||,  /* the message to log */
           '" in "'||Location||'"; folder may already exist'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
 Success = 0  /* the result is false */
end

return Success  /* end of MakeFolder */

/**********************************************************************/
MakeObject: procedure expose LogFile  /* creates a program object */
/**********************************************************************/

parse arg Title,Location,ExeName,Parameters,AssocType,ObjectID,IconFile,AddSettings  /* get the arguments */

Settings = 'EXENAME='||ExeName||';'||,  /* include the executable path and name */
           'PARAMETERS='||Parameters||';'||,  /* include the executable parameters */
           'ASSOCTYPE='||AssocType||';'||,  /* include the association type(s) */
           'OBJECTID='||ObjectID||';'||,  /* include the object ID */
           'ICONFILE='||IconFile||';'||,  /* inlcude the icon file path and name */
           AddSettings  /* include any additional settings */

if (syscreateobject('WPProgram',Title,Location,Settings)) then  /* if we can create the object */
do
 Message = 'Created object with ObjectID "'||ObjectID||,  /* the message to log */
          '" and title "'||Title||,  /* the message to log */
          '" in "'||Location||'"'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
end
else  /* if we cannot create the object */
do
 Message = 'ERROR: Could not create object with ObjectID "'||ObjectID||,  /* the message to log */
           '" and title "'||Title||,  /* the message to log */
           '" in "'||Location||'"; object may already exist '  /* the message to log */
 call WriteLog LogFile,Message  /* report */
end

return 0  /* end of MakeObject */

/**********************************************************************/
MakeShadow: procedure expose LogFile  /* creates a shadow */
/**********************************************************************/

parse arg Name,Location  /* get the arguments */

if (syscreateshadow(Name,Location)) then  /* if we can create the shadow */
do
 Message = 'Created shadow of "'||Name||,  /* the message to log */
           '" in "'||Location||'"'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
end
else  /* if we cannot create the shadow */
do
 Message = 'ERROR: Could not create shadow of "'||Name||,  /* the message to log */
           '" in "'||Location||'"; shadow may already exist'  /* the message to log */
 call WriteLog LogFile,Message  /* report */
end

return 0  /* end of MakeShadow */

/**********************************************************************/
WriteLog: procedure  /* writes a log entry to a log file */
/**********************************************************************/

parse arg LogFile,Text  /* get the arguments */

if (stream(LogFile,'C','OPEN WRITE') >< 'READY:') then  /* if we cannot open the log file for writing */
do
 say 'FATAL ERROR: Could not write to log file'  /* show a message */
 call Halt  /* and abort */
end

call lineout LogFile,Text  /* write the text to the file */
call stream LogFile,'C','CLOSE'  /* close the file */

return 0  /* end of WriteLog */

/**********************************************************************/
Explain:  /* Explain to the user what's going to happen */
/**********************************************************************/

call ClearScreen  /* clear the screen */

say 'This program will ask you for a main directory name. The directory will be'  /* screen info */
say 'optionally created if it does not exist. The main directory will hold the mail'  /* screen info */
say 'folders (In, Out, In Archive, Out Archive) and the Addresses, Accessories, and'  /* screen info */
say 'Configuration folders. Some file objects and object shadows will be created'  /* screen info */
say 'in these folders (the objects themselves are in the "Objects" subfolder of the'  /* screen info */
say 'RexxMail program directory -- the directory from which you are running this'  /* screen info */
say 'program). The RexxMail program files will remain in the directory in which'  /* screen info */
say 'you unzipped them. Apart from defining the file-program associations needed to'  /* screen info */
say 'make the operating system recognize mail message files, this program will not'  /* screen info */
say 'change any system files (e.g. CONFIG.SYS), nor will it install any DLLs.'  /* screen info */
say 'The installation program will also create a folder on your desktop with some'  /* screen info */
say 'useful shadows of program and file objects.'  /* screen info */
say '_______________________________________________________________________________'  /* line */
say ''  /* empty line */
say 'All the changes made by the installation program will be written to a log file,'  /* screen info */
say '"install.log" in the program directory, for your information.'  /* screen info */
say '_______________________________________________________________________________'  /* separator */
say ''  /* empty line */
say 'You can press [Ctrl]+[C] at any time to abort the installation'  /* screen info */
say '_______________________________________________________________________________'  /* separator */
say ''  /* empty line */
call charout StdOut,'Continue? [Y/N] Y'||d2c(8)  /* user prompt */
InKey = sysgetkey('ECHO')  /* get a response */
say ''  /* skip to the next line */
if (pos(InKey,'Yy+'||d2c(13)) = 0) then  /* if it is not Y(es) or [Enter] */
do
 call Halt  /* abort */
end

return 0  /* end of Explain */

/**********************************************************************/
ClearScreen: procedure  /* clears the screen ("syscls" does not preserve screen colours) */
/**********************************************************************/

ScreenLines = word(systextscreensize(),1)  /* get the number of screen lines */
do ScreenLines  /* repeat as many times as we have screen lines */
 say ''  /* write an empty line to clear the screen */
end

say '_______________________________________________________________________________'  /* line */
say 'RexxMail installation _________________________________________________________'  /* screen info */
say ''  /* empty line */

return  /* end of ClearScreen */

/**********************************************************************/
LoadRexxUtils: procedure  /* loads REXX utility functions if necessary */
/**********************************************************************/

Success = 1  /* all is well to start with */

if (rxfuncquery('SysLoadFuncs') >< 0) then  /* if we have to load the REXX utility functions */
do
 if (rxfuncadd('SysLoadFuncs','RexxUtil','SysLoadFuncs') = 0) then  /* if we can register the general loading function */
 do
  call sysloadfuncs  /* call the general loading function */
 end
 else  /* if we cannot register the general loading function */
 do
  Message = 'Cannot load Rexx utility functions'  /* report */
  call WriteLog LogFile,Message  /* report */
  Success = 0  /* no success */
 end
end

return Success  /* end of LoadRexxUtils */

/**********************************************************************/
InstallOK:  /* we end up here if everything installed OK */
/**********************************************************************/

Message = date('S')||' '||time('N')||,  /* include a date and time stamp */
          ' : Installation completed'  /* the message to log */
call WriteLog LogFile,Message  /* report */

call ClearScreen  /* clear the screen */

say 'The following combination has been added to "'||LocFile||'" :'  /* report */
say ''  /* empty line */
say '  User name   : "'||UserName||'"'  /* report the user name */
say '  Main folder : "'||MailLoc||'"'  /* report the main folder location */
say ''  /* empty line */
say ''  /* empty line */
say 'For further instructions, please refer to section 6, "What to do after'  /* report */
say 'installation", of the readme.txt file in the RexxMail program directory.'  /* report */

call Quit  /* and quit */

/*****************************************************************/
Syntax:  /* we end up here if a syntax error occurred */
/*****************************************************************/

Message = date('S')||' '||time('N')||,  /* include a date and time stamp */
          ' : A syntax error occurred in line '||sigl||,  /* include the line with the error */
          ' of the installation program;'||,  /* more text */
          d2c(13)||d2c(10)||,  /* start a new line */
          copies(' ',20),  /* insert 20 spaces */
          '                    please contact the author at <marcus@degeus.com>'  /* end of the message to log */
call WriteLog LogFile,Message  /* report */

/*****************************************************************/
Halt:  /* we end up here if a halt condition occurred */
/*****************************************************************/

call beep 444,111  /* signal */
call beep 333,222  /* signal */

Message = date('S')||' '||time('N')||,  /* include a date and time stamp */
          ' : Installation aborted'  /* end of the message to log */
call WriteLog LogFile,Message  /* report */

call ClearScreen  /* clear the screen */

say 'Aborted'  /* show a message */

/*****************************************************************/
Quit:  /* we end up here at the end of it all */
/*****************************************************************/

say ''  /* empty line */
say ''  /* empty line */
say 'For details of the installation, see the install.log file in the'  /* report */
say 'RexxMail program directory.'  /* report */

say ''  /* empty line */
say ''  /* empty line */
call charout StdOut,'End of RexxMail installation - press [Enter] to exit'  /* report */

parse pull  /* wait for [Enter] */

exit  /* and exit */
